<?php

namespace Bixo\Asset\Http\Controllers;

use Exception;
use Bixo\Asset\Actions\AssetWorkflow;
use Bixo\Asset\Http\Requests\AssetWorkflowRequest;
use Bixo\Asset\Http\Resources\AssetResource;
use Bixo\Asset\Models\Asset;
use Litepie\Http\Controllers\ActionController as BaseController;

/**
 * Workflow controller class.
 *
 */
class AssetWorkflowController extends BaseController
{
    /**
     * Action controller function for asset.
     *
     * @param Model $asset
     * @param action next action for the asset.
     *
     * @return Response
     */
    public function __invoke(AssetWorkflowRequest $request, Asset $asset, $trasition)
    {
        try {
            $request = $request->all();
            $asset = AssetWorkflow::run($trasition, $asset, $request);
            $data = new AssetResource($asset);
            return $this->response->message(trans('messages.success.updated', ['Module' => trans('asset::asset.name')]))
                ->code(204)
                ->data(compact('data'))
                ->status('success')
                ->url(guard_url('asset/asset/' . $asset->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('asset/asset/' . $asset->getRouteKey()))
                ->redirect();
        }
    }


    /**
     * Get the asset and user for the workflow.
     *
     * @param PublicRequest $request
     * @param string $asset
     * @param string $user
     *
     * @return Response
     */
    public function get(PublicRequest $request, $asset, $user)
    {
        if (!$request->hasValidSignature()) {
            abort(403, 'This url is expired!');
        }

        $auth = Auth::guard()->getProvider()->getModel();
        $user = $auth::findBySignedId($user);
        Auth::guard()->login($user);
        $asset = Asset::findBySignedId($asset);
        $transitions = $this->transitions($asset, $user);
        $user = user();
        Auth::logout();
        return $this->response->setMetaTitle(trans('asset::asset.name') . ' ' . trans('Approvals'))
            ->view('asset::public.asset.workflow')
            ->layout('mobile')
            ->data(compact('asset', 'user', 'transitions'))
            ->output();
    }

    /**
     * Display the workflow form for the asset.
     *
     * @param PublicRequest $request
     * @param string $asset
     * @param string $user
     *
     * @return Response
     */
    public function post(PublicRequest $request, $asset, $user)
    {
        if (!$request->hasValidSignature()) {
            abort(403, 'This url is expired!');
        }
        $transition = $request->transition;
        $auth = Auth::guard()->getProvider()->getModel();
        $user = $auth::findBySignedId($user);
        Auth::guard()->login($user);
        $asset = Asset::findBySignedId($asset);
        $user = user();
        $asset = AssetWorkflow::run($transition, $asset, $request->all());
        Auth::logout();

        return response()->json(
            [
                'status' => 'success',
                'url' => $request->url(),
            ]
        );
    }

    private function transitions($asset, $user)
    {
        $transitions = [];
        foreach ($asset->workflow()->transitions($asset) as $key => $value) {
            $name = $value->getName();
            $array['url'] = URL::temporarySignedRoute('litecms.asset.workflow', now()->addMinutes(3), [
                'guard' => 'admin',
                'transition' => $name,
                'asset' => $asset->getSignedId(now()->addMinutes(3)),
                'user' => $user->getSignedId(now()->addMinutes(3)),
                'trans' => 'en',
            ]
            );
            $array['name'] = $name;
            $array['key'] = $name;
            $array['form'] = $asset->workflow()->form($value);
            $array['label'] = trans('asset::asset.workflow.' . $name);
            $transitions[] = $array;
        }
        return $transitions;
    }
}
